#include <Arduino.h>
#include <BluetoothSerial.h>
#include <esp_now.h>
#include <WiFi.h>

BluetoothSerial Serial_BT;

// Motor 3
int r_en3 = 15; // +
int l_en3 = 0; // -
int r_pwm3 = 2; // +
int l_pwm3 = 4; // -

// Motor 4
int r_en4 = 16; // +
int l_en4 = 18; // -
int r_pwm4 = 17; // +
int l_pwm4 = 19; // -

//Ultrasonic Sensor Pinouts

int trigPin = 13;
int echoPin = 12;

int distance;
long duration;

int incoming;

uint8_t slaveAddress[] = {0xC8, 0x2E, 0x18, 0x67, 0x36, 0x6C}; // Replace with your slave ESP32 MAC address

void onDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
  if (status != ESP_NOW_SEND_SUCCESS) {
    Serial.println("Failed to send data");
  }
}

void setup() {
  //motor driver;
  pinMode(r_en3, OUTPUT);
  pinMode(l_en3, OUTPUT);
  pinMode(r_pwm3, OUTPUT);
  pinMode(l_pwm3, OUTPUT);
  pinMode(r_en4, OUTPUT);
  pinMode(l_en4, OUTPUT);
  pinMode(r_pwm4, OUTPUT);
  pinMode(l_pwm4, OUTPUT);

  Serial.begin(115200);
  Serial_BT.begin("ESP32Test");
  Serial.println("The device has started, now you can pair it with Bluetooth");

  // Initialize WiFi and ESP-NOW
  WiFi.mode(WIFI_STA);
  WiFi.disconnect(); // Ensure no connections interfere with ESP-NOW

  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  esp_now_register_send_cb(onDataSent);


  //Register Peer
  esp_now_peer_info_t peerInfo;
  memcpy(peerInfo.peer_addr, slaveAddress, 6);
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;
  
  if (esp_now_add_peer(&peerInfo) != ESP_OK) {
    Serial.println("Failed to add peer");
    return;
  } else {
    Serial.println("Peer added successfully");
  }
}

void loop() {
  while (Serial_BT.available()) {
    String Direction = Serial_BT.readStringUntil('\n');
    Serial.print("Received Direction: ");
    Serial.println(Direction);

    esp_err_t result = esp_now_send(slaveAddress, (uint8_t *)Direction.c_str(), Direction.length());
    if (result == ESP_OK) {
      Serial.println("ESP-NOW message sent successfully");
    } else {
      Serial.print("Error sending ESP-NOW message: ");
      Serial.println(result);
    }

    if (Direction == "F") {
      // Forward
      moveForward();
    } else if (Direction == "B") {
      // Backward
      moveBackward();   
    } else if (Direction == "L") {
      // Left
      moveLeft();
    } else if (Direction == "R") {
      // Right
      moveRight();
    } else if (Direction == "FL") {
      // Forward Left
      moveForwardLeft();
    } else if (Direction == "FR") {
      // Forward Right
      moveForwardRight();
    } else if (Direction == "BL") {
      // Backward Left
      moveBackwardLeft();
    } else if (Direction == "BR") {
      // Backward Right
      moveBackwardRight();
    } else if (Direction == "S") {
      // STOP
      moveStop();
    }
  delay(1000);
  }
  //Ultrasonic Sensor Reading
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  duration = pulseIn(echoPin, HIGH);
  distance = duration * 0.034 / 2;

  if (distance < 20) { // Distance threshold in centimeters
    Serial_BT.write('O');  // 'O' for obstacle detected
    delay(500); // Add a delay to avoid constant triggering
 }
}

void moveForward() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 255);
  analogWrite(l_pwm3, 0);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 255);
  analogWrite(l_pwm4, 0);
}

void moveBackward() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 0);
  analogWrite(l_pwm3, 255);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 0);
  analogWrite(l_pwm4, 255);
}

void moveLeft() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 255);
  analogWrite(l_pwm3, 0);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 0);
  analogWrite(l_pwm4, 255);
}

void moveRight() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 0);
  analogWrite(l_pwm3, 255);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 255);
  analogWrite(l_pwm4, 0);
}

void moveForwardLeft() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 255);
  analogWrite(l_pwm3, 0);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 0);
  analogWrite(l_pwm4, 0);
}

void moveForwardRight() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 0);
  analogWrite(l_pwm3, 0);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 255);
  analogWrite(l_pwm4, 0);
}

void moveBackwardLeft() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 0);
  analogWrite(l_pwm3, 0);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 0);
  analogWrite(l_pwm4, 255);
}

void moveBackwardRight() {

  digitalWrite(r_en3, HIGH);
  digitalWrite(l_en3, HIGH);
  analogWrite(r_pwm3, 0);
  analogWrite(l_pwm3, 255);
  digitalWrite(r_en4, HIGH);
  digitalWrite(l_en4, HIGH);
  analogWrite(r_pwm4, 0);
  analogWrite(l_pwm4, 0);
}

void moveStop() {

  digitalWrite(r_en3, LOW);
  digitalWrite(l_en3, LOW);
  analogWrite(r_pwm3, 0);
  analogWrite(l_pwm3, 0);
  digitalWrite(r_en4, LOW);
  digitalWrite(l_en4, LOW);
  analogWrite(r_pwm4, 0);
  analogWrite(l_pwm4, 0);
}
